"""
Create by yy on 2019-07-25
"""
import threading
from concurrent.futures import ThreadPoolExecutor

from tool_yy import debug


class HelperTest(object):
    def __init__(self, init_db=None):
        self.db = init_db("INSOMNIA_MUSIC_DATABASE_CONFIG")

    def __del__(self):
        self.db.close()


def thread_func(info):
    print(threading.current_thread().name)
    print(info)


if __name__ == "__main__":
    with ThreadPoolExecutor(max_workers=5) as thread_pool:
        for i in range(10):
            thread_pool.submit(thread_func, i)



"""
author songjie
"""
from .config import Config
from .helper_context import HelperContext
from .db import DBConfig


class HelperConfig(HelperContext):
    config_class = Config
    DEFAULT_CONFIG = {
        "MYSQL_DATABASE": "mysql",
        "MYSQL_USERNAME": "root",
        "MYSQL_PASSWORD": "root",
        "MYSQL_HOST": "127.0.0.1",
        "MYSQL_PORT": "3306",
        "MYSQL_TABLE_PREFIX": ""
    }

    def __init__(self, **kwargs):
        super().__init__()
        self._root_path = None
        self.config = self.make_config()
        self._db = DBConfig()

    def __del__(self):
        with self.auto_handle_exception():
            self._db.close()

    @property
    def db(self):
        return self._db

    def init_db(self, config=None):
        """
        初始化数据库设置
        init db config
        :param config:
        :return:
        """
        if not config:
            config = "MYSQL_CONFIG"
        self._db = DBConfig(
            username=self.config[config].setdefault("MYSQL_USERNAME", HelperConfig.DEFAULT_CONFIG["MYSQL_USERNAME"]),
            password=self.config[config].setdefault("MYSQL_PASSWORD",
                                                    HelperConfig.DEFAULT_CONFIG["MYSQL_PASSWORD"]),
            database=self.config[config].setdefault("MYSQL_DATABASE",
                                                    HelperConfig.DEFAULT_CONFIG["MYSQL_DATABASE"]),
            host=self.config[config].setdefault("MYSQL_HOST", HelperConfig.DEFAULT_CONFIG["MYSQL_HOST"]),
            port=self.config[config].setdefault("MYSQL_PORT", HelperConfig.DEFAULT_CONFIG["MYSQL_PORT"]),
            table_prefix=self.config[config].setdefault("MYSQL_TABLE_PREFIX",
                                                        HelperConfig.DEFAULT_CONFIG["MYSQL_TABLE_PREFIX"])
        )
        return self._db

    def make_config(self):
        """
        构造配置项
        :return:
        """
        root_path = self._root_path
        return self.config_class(root_path=root_path, defaults=HelperConfig.DEFAULT_CONFIG)



import codecs
from itsdangerous import json as _json


def loads(s, **kwargs):
    """Deserialize an object from a JSON-formatted string ``s``. If
    there is an app context pushed, use the current app's configured
    decoder (:attr:`~flask.Flask.json_decoder`), or fall back to the
    default :class:`JSONDecoder`.

    Takes the same arguments as the built-in :func:`json.loads`, and
    does some extra configuration based on the application. If the
    simplejson package is installed, it is preferred.

    :param s: JSON string to deserialize.
    :param app: App instance to use to configure the JSON decoder.
        Uses ``current_app`` if not given, and falls back to the default
        encoder when not in an app context.
    :param kwargs: Extra arguments passed to :func:`json.dumps`.

    .. versionchanged:: 1.0.3

        ``app`` can be passed directly, rather than requiring an app
        context for configuration.
    """
    if isinstance(s, bytes):
        encoding = kwargs.pop('encoding', None)
        if encoding is None:
            encoding = detect_encoding(s)
        s = s.decode(encoding)
    return _json.loads(s, **kwargs)


def detect_encoding(data):
    """Detect which UTF codec was used to encode the given bytes.

    The latest JSON standard (:rfc:`8259`) suggests that only UTF-8 is
    accepted. Older documents allowed 8, 16, or 32. 16 and 32 can be big
    or little endian. Some editors or libraries may prepend a BOM.

    :param data: Bytes in unknown UTF encoding.
    :return: UTF encoding name
    """
    head = data[:4]

    if head[:3] == codecs.BOM_UTF8:
        return 'utf-8-sig'

    if b'\x00' not in head:
        return 'utf-8'

    if head in (codecs.BOM_UTF32_BE, codecs.BOM_UTF32_LE):
        return 'utf-32'

    if head[:2] in (codecs.BOM_UTF16_BE, codecs.BOM_UTF16_LE):
        return 'utf-16'

    if len(head) == 4:
        if head[:3] == b'\x00\x00\x00':
            return 'utf-32-be'

        if head[::2] == b'\x00\x00':
            return 'utf-16-be'

        if head[1:] == b'\x00\x00\x00':
            return 'utf-32-le'

        if head[1::2] == b'\x00\x00':
            return 'utf-16-le'

    if len(head) == 2:
        return 'utf-16-be' if head.startswith(b'\x00') else 'utf-16-le'

    return 'utf-8'



"""
Create by yy on 2019/9/23
"""
INSOMNIA_MUSIC_DATABASE_CONFIG = {
    "MYSQL_DATABASE": "insomnia_music",
    "MYSQL_USERNAME": "yy",
    "MYSQL_PASSWORD": "wyysdsa!",
    "MYSQL_HOST": "127.0.0.1",
    "MYSQL_PORT": "3306"
}


# -*- coding:utf8 -*-

from phoenixdb import connect
from .function import debug
import pymysql
from tool_yy.config import dbconfig as pymysql_config_origin, phoenix_db_config as dbconfig


# noinspection PyPep8Naming,PyMethodMayBeStatic,PyBroadException,PyStatementEffect
class DBConfig(object):
    """
    PS: If you want to use this tool to insert data,
        you should start your localhost mysql,
        because I can't filter some str,
        so I use pymysql's method which called 'escape'.
        And if you want to use it,
        now, you only can start your localhost mysql.
        If someone can solve it, welcome to leave a message for me.
        Thanks.
    """
    config = dbconfig.dbconfig()
    pymysql_config = pymysql_config_origin.dbconfig()
    host = config['host']
    table_prefix = config['table_prefix']
    is_connection = False
    try:
        is_debug = config['debug']
    except:
        is_debug = False
    db = None
    cursor = None

    def __init__(self):
        pass

    def getConnect(self):
        return connect(url=self.host, autocommit=True)

    def getCursor(self):
        try:
            if not self.is_connection:
                self.db = self.getConnect()
                self.is_connection = True
        except Exception as e:
            if self.is_debug:
                debug("数据库连接失败：")
                debug(e)
        return self.db.cursor()

    def closeDB(self):
        try:
            self.db.close()
            self.is_connection = False
        except Exception as e:
            if self.is_debug:
                debug("数据库关闭失败：")
                debug(e)

    def select(self, data, get_all=True, is_close_db=True):
        self.cursor = self.getCursor()
        sql = self.getSelectSql(data)
        try:
            data['columns'][0]
            if data['columns'][0] == "*":
                try:
                    columns_sql = {
                        "table": data['table']
                    }
                    columns_data = self.getColumns(columns_sql)
                except:
                    self.closeDB()
                    columns_data = list()
                    if self.is_debug:
                        return {"error": "字段信息获取失败"}
            else:
                columns_data = tuple()
                for v in data['columns']:
                    tmp_tuple = ((v,),)
                    columns_data = columns_data + tmp_tuple
        except:
            try:
                columns_sql = {
                    "table": data['table']
                }
                columns_data = self.getColumns(columns_sql)
            except:
                self.closeDB()
                if self.is_debug:
                    return {"error": "字段信息获取失败"}
        try:
            self.cursor.execute(sql)
            if get_all:
                results = self.cursor.fetchall()
            else:
                results = self.cursor.fetchone()
        except:
            self.closeDB()
            if self.is_debug:
                results = list()
                return {"error": "数据获取失败"}
        if is_close_db:
            self.closeDB()
        if get_all:
            results_final = list()
            for v in results:
                length = len(v)
                content = dict()
                for k in range(length):
                    content[columns_data[k][0]] = v[k]
                results_final.append(content)
        else:
            results_final = dict()
            length = len(results)
            for k in range(length):
                results_final[columns_data[k][0]] = results[k]
        return results_final

    def getColumns(self, data, num=1, is_close_db=False):
        self.cursor = self.getCursor()
        data['limit'] = [0, 1]
        sql = self.getSelectSql(data)
        try:
            self.cursor.execute(sql)
            if num == 1:
                results = self.cursor.description
            else:
                results = self.cursor.description
        except Exception as e:
            results = 0
            if self.is_debug:
                debug("It's error that get table columns")
                debug(e)
        if is_close_db:
            self.closeDB()
        return results

    def insert(self, sql, is_close_db=True, table_columns=False):
        self.cursor = self.getCursor()
        table = sql['table']
        del sql['table']
        sql = self.getInsertSql(sql, table, self_colums=table_columns)
        try:
            self.cursor.execute(sql)
            self.db.commit()
            results = 1
        except Exception as e:
            if self.is_debug:
                debug("Database insert error, error info is:")
                debug(e)
            results = 0
        if is_close_db:
            self.closeDB()
        return results

    def insertLastId(self, sql, is_close_db=True):
        self.cursor = self.getCursor()
        try:
            self.cursor.execute(sql)
            self.db.commit()
            results = self.cursor.lastrowid
        except Exception as e:
            if self.is_debug:
                debug("Database insert error")
                debug(e)
            results = 0
        if is_close_db:
            self.closeDB()
        return results

    def update(self, sql, is_close_db=True):
        self.cursor = self.getCursor()
        sql = self.getUpdateSql(sql)
        debug(sql)
        try:
            self.cursor.execute(sql)
            self.db.commit()
            results = 1
        except Exception as e:
            if self.is_debug:
                debug("Database update error")
                debug(e)
            results = 0
        if is_close_db:
            self.closeDB()
        return results

    def delete(self, data, is_close_db=True):
        self.cursor = self.getCursor()
        sql = self.getDeleteSql(data)
        try:
            self.cursor.execute(sql)
            self.db.commit()
            results = 1
        except Exception as e:
            if self.is_debug:
                debug("Database delete error")
                debug(e)
            results = 0
        if is_close_db:
            self.closeDB()
        return results

    # noinspection PyUnresolvedReferences
    def getDeleteSql(self, data, is_close_db=False):
        sql = "delete "
        s = ""
        sql = sql + s + ' from "' + self.table_prefix + data['table'] + '"'
        # if there is a condition , we spell it
        try:
            unusedata = data['condition']
            sql = sql + " where "
            s = ""
            for i in data['condition']:
                s = s + i + " "
            sql = sql + s
        except:
            pass
        # if there is a order need , we spell it
        try:
            data['order']
            sql = sql + " order by " + data['order'][0] + " " + data['order'][1]
        except:
            pass
        # if there is limit. we spell it
        try:
            data['limit']
            sql = sql + " limit " + str(data['limit'][0]) + "," + str(data['limit'][1])
        except:
            pass
        if is_close_db:
            self.closeDB()
        return sql

    # noinspection PyUnresolvedReferences
    def getSelectSql(self, data, is_close_db=False):
        sql = "select "
        s = ""
        # spell condition
        try:
            for i in data['columns']:
                if i != data['columns'][-1]:
                    s = s + "\"" + i + "\"" + ","
                else:
                    s = s + "\"" + i + "\""
        except:
            s = "*"
        sql = sql + s + ' from "' + self.table_prefix + data['table'] + '"'
        # if there is a condition , we spell it
        try:
            data['condition']
            sql = sql + " where "
            s = ""
            for i in data['condition']:
                s = s + i + " "
            sql = sql + s
        except:
            pass
        # if there is a order need , we spell it
        try:
            data['order']
            sql = sql + " order by " + data['order'][0] + " " + data['order'][1]
        except:
            pass
        # if there is limit. we spell it
        try:
            data['limit']
            sql = sql + " limit " + str(data['limit'][1]) + " offset " + str(data['limit'][0])
        except:
            pass
        if is_close_db:
            self.closeDB()
        return sql

    def getUpdateSql(self, data, is_close_db=False):
        sql = "update "
        sql = sql + '"' + self.table_prefix + data['table'] + '" set '
        try:
            data['set']
            s = ""
            for i in data['set']:
                s = s + i + " "
            sql = sql + s
        except:
            pass
        # if there is a condition , we spell it
        try:
            data['condition']
            sql = sql + "where "
            s = ""
            for i in data['condition']:
                s = s + i + " "
            sql = sql + s
        except:
            pass
        if is_close_db:
            self.closeDB()
        return sql

    def getInsertSql(self, data, table, is_close_db=False, self_colums=False):
        # 构造插入查询语句，此函数传入参数data必须为dict()类型
        s = "upsert into \"" + self.table_prefix + table + "\"("
        columns = ""
        value = ""
        # get table's columns name
        table_sql = {
            "table": table
        }
        if not self_colums:
            table_columns = self.getColumns(table_sql)
        else:
            table_columns = self_colums
        table_columns_dict = dict()
        str_dict = {"text": "text", "varchar": "varchar", "longtext": "longtext", "datetime": "datetime",
                    "char": "char"}
        int_dict = {"int": "int", "bigint": "bigint", "decimal": "decimal", "double": "double", "float": "float",
                    "integer": "integer"}
        try:
            for v in table_columns:
                table_columns_dict[v[0]] = v[1].lower()
        except:
            pass
        length = len(data)
        i = 1
        filter_db = pymysql.connect(self.pymysql_config['host'], self.pymysql_config['username'],
                                    self.pymysql_config['password'], self.pymysql_config['database'], charset="utf8")
        for k, v in data.items():
            if i != length:
                columns = columns + '"' + k + '",'
                # if k.lower() == "id":
                if table_columns_dict[k.lower()] in int_dict:
                    if v == "":
                        v = 0
                    tmpstr = "%s," % str(v)
                else:
                    tmpstr = "%s," % filter_db.escape(v)
                value = value + tmpstr
            else:
                columns = columns + '"' + k + '"'
                if table_columns_dict[k.lower()] in int_dict:
                    if v == "":
                        v = 0
                    tmpstr = "%s" % str(v)
                else:
                    tmpstr = "%s" % filter_db.escape(v)
                value = value + tmpstr
            if k in table_columns_dict:
                del table_columns_dict[k]
            i = i + 1
        filter_db.close()
        for k, v in table_columns_dict.items():
            columns = columns + ",\"" + str(k) + "\""
            if table_columns_dict[k] in str_dict:
                value = value + ",'" + str("") + "'"
            else:
                value = value + "," + "0"
        data = s + columns + ")values(" + value + ")"
        if is_close_db:
            self.closeDB()
        return data

    def createTable(self, sql, sequence=False):
        self.cursor = self.getCursor()
        try:
            self.cursor.execute(sql)
            if isinstance(sequence, str):
                self.cursor.execute("create sequence " + sequence)
            if self.is_debug:
                debug("数据库创建成功")
        except Exception as e:
            if self.is_debug:
                debug("创建数据库出错，报错信息：")
                debug(e)
        finally:
            self.closeDB()
        return 0

    def free(self, sql, is_close_db=True):
        self.cursor = self.getCursor()
        data = 1
        try:
            self.cursor.execute(sql)
            if sql.lower().find("select ") != -1:
                data = self.cursor.fetchall()
        except Exception as e:
            if self.is_debug:
                debug("原生语句执行出错，报错信息：")
                data = 0
                debug(e)
        finally:
            if is_close_db:
                self.closeDB()
        return data


"""
designed by yy(奕弈)
"""



import datetime
import json
import hashlib
import requests
import random
import time

# change the dict to a json str
# noinspection PyShadowingBuiltins
from tool_yy.config import settings


def js_arr(text=None, id=None, reply=None):
    """
    This function is a tool which change the dict to a json str

    :param text: any type
    :param id: any type
    :param reply: any type
    :return: str
    """
    data = {'text': text, "id": id, 'reply': reply}
    return json.dumps(data)


# get a md5 str
def md5(s1):
    """
    This function can turn a str into a str which has been encrypted by md5

    :param s1: It's a str.
    :return: str
    """
    s = str(s1)
    h1 = hashlib.md5()
    h1.update(s.encode(encoding='utf-8'))
    s = h1.hexdigest()
    return s


# get a sha1 str
def sha1(s1):
    """
        This function can turn a str into a str which has been encrypted by md5

        :param s1: It's a str.
        :return: str
        """
    s = str(s1)
    h1 = hashlib.sha1()
    h1.update(s.encode(encoding='utf-8'))
    s = h1.hexdigest()
    return s


# debug test
# noinspection PyRedundantParentheses,PyBroadException
def change_to_str(data, rowstr="<br>", count=4, origin_count=4):
    """
    This function can turn a data which you give into a str which you can look easily.
    Like a dict {"text": 1, "id":2, "reply":3}
    call the function changeToStr(text)
    you will look follow data:
    dict(3) =>{
      ["text"] => 1
      ["id"] => 2
      ["reply"] => 3
    }

    :param data: data you give, it can be any type.
    :param rowstr: it's a str. Enter, if you want to show data in web, it's default,
    or you want to show data in console, you should set rowstr="\n"
    :param count: It's a int. spacing num(I'm so sorry, I have forget the parameter's meaning, but you can try changing it.)
    :param origin_count: It's a int. spacing num(I'm so sorry, I have forget the parameter's meaning)
    :return: str
    """
    s = ""
    space1 = rowstr
    space2 = rowstr
    if count == 0:
        endstr = "}"
    else:
        endstr = "}"
    for i in range(count):
        space1 = space1 + " "
        if i >= origin_count:
            space2 = space2 + " "
    count = count + origin_count
    if isinstance(data, dict):
        length = len(data)
        s = s + "dict(" + str(length) + ") => {"
        for k, v in data.items():
            s = s + space1 + "['" + str(k) + "'] => " + change_to_str(v, rowstr, count, origin_count)
        s = s + endstr if not length else s + space2 + endstr
    elif isinstance(data, (tuple)):
        length = len(data)
        s = s + "tuple(" + str(length) + ") => {"
        i = 0
        for v in data:
            s = s + space1 + "[" + str(i) + "] => " + change_to_str(v, rowstr, count, origin_count)
            i = i + 1
        s = s + ")" if not length else s + space2 + ")"
    elif isinstance(data, (list)):
        length = len(data)
        s = s + "list(" + str(length) + ") => ["
        i = 0
        for v in data:
            s = s + space1 + "[" + str(i) + "] => " + change_to_str(v, rowstr, count, origin_count)
            i = i + 1
        s = s + "]" if not length else s + space2 + "]"
    else:
        s = str(data)
    return s


def format_data(data):
    """
    :param data:
    :return: str : this str can print to web page.
    """
    return "<pre>" + change_to_str(data, rowstr="<br/>") + "</pre>"


def debug(data="", is_set_time=settings.DEBUG_TIME):
    """
    :param is_set_time:
    :param data:
    :return: no return
    """
    if is_set_time:
        now_time = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        print("[" + now_time + "]  " + change_to_str(data, rowstr="\n"))
    else:
        print(change_to_str(data, rowstr="\n"))


def replace_html(s):
    """
    :param s: str
    :return: str
    """
    s = s.replace('&quot;', '"')
    s = s.replace('&amp;', '&')
    s = s.replace('&lt;', '<')
    s = s.replace('&gt;', '>')
    s = s.replace('&nbsp;', ' ')
    s = s.replace(r'\/', '/')
    return s


# noinspection PyBroadException
def curl_data(url, value=False, referer=False, cookie=False, header=None, proxy_ip="", timeout=50,
              open_virtual_ip=False, params=None, return_response=False, allow_redirects=True):
    """
    This function can get a web page's source data.

    :param allow_redirects: (optional) Boolean. Enable/disable GET/OPTIONS/POST/PUT/PATCH/DELETE/HEAD redirection.
    Defaults to ``True``.
    :param return_response:
    :param params:
    :param open_virtual_ip:
    :param timeout:
    :param proxy_ip:
    :param header:
    :param url: str
    :param value: dict or default(None)
    :param referer: str
    :param cookie: str
    :return: str(web page's source data)
    """
    if params is None:
        params = {}
    if header is None:
        header = dict()
    headers = dict()
    headers['User-Agent'] = "baiduspider"
    headers['Accept'] = "*/*"
    headers['Connection'] = "keep-alive"
    if open_virtual_ip:
        ip = virtual_ip()
        headers['CLIENT-IP'] = ip
        headers['X-FORWARDED-FOR'] = ip
    if isinstance(cookie, str):
        headers['Cookie'] = cookie
    if isinstance(referer, str):
        headers['Referer'] = referer
    headers = headers.copy()
    headers.update(header)
    if proxy_ip != "":
        proxy_ip_dict = {
            "http": proxy_ip,
            "https": proxy_ip
        }
    else:
        proxy_ip_dict = dict()
    if isinstance(value, dict):
        if isinstance(cookie, dict):
            res = requests.post(url, data=value, headers=headers, proxies=proxy_ip_dict, cookies=cookie,
                                timeout=timeout, allow_redirects=allow_redirects)
        else:
            res = requests.post(url, data=value, headers=headers, proxies=proxy_ip_dict, timeout=timeout,
                                allow_redirects=allow_redirects)
    elif isinstance(params, dict):
        if isinstance(cookie, dict):
            res = requests.get(url, params=value, headers=headers, proxies=proxy_ip_dict, cookies=cookie,
                               timeout=timeout, allow_redirects=allow_redirects)
        else:
            res = requests.get(url, params=value, headers=headers, proxies=proxy_ip_dict, timeout=timeout,
                               allow_redirects=allow_redirects)
    else:
        if isinstance(cookie, dict):
            res = requests.get(url, headers=headers, proxies=proxy_ip_dict, cookies=cookie, timeout=timeout,
                               allow_redirects=allow_redirects)
        else:
            res = requests.get(url, headers=headers, proxies=proxy_ip_dict, timeout=timeout,
                               allow_redirects=allow_redirects)
    try:
        data = res.content.decode("utf-8")
    except UnicodeDecodeError:
        try:
            data = res.content.decode("GBK")
        except UnicodeDecodeError:
            data = res.content
    return (data, res) if return_response else data


# noinspection PyBroadException
def get_cookie(url, value=False, referer=False, cookie=False, header=None, proxy_ip="", timeout=50,
               open_virtual_ip=False, params=None, allow_redirects=True):
    """
        This function can get a web page's source data.

        :param allow_redirects: (optional) Boolean. Enable/disable GET/OPTIONS/POST/PUT/PATCH/DELETE/HEAD redirection.
        Defaults to ``True``.
        :param params:
        :param timeout:
        :param open_virtual_ip:
        :param proxy_ip:
        :param header:
        :param url: str
        :param value: dict or default(None)
        :param referer: str
        :param cookie: str
        :return: str(web page's source data)
        """
    if header is None:
        header = {}
    headers = dict()
    headers['User-Agent'] = "baiduspider"
    headers['Accept'] = "*/*"
    headers['Connection'] = "keep-alive"
    if isinstance(cookie, str):
        headers['Cookie'] = cookie
    if isinstance(referer, str):
        headers['Referer'] = referer
    if open_virtual_ip:
        ip = virtual_ip()
        headers['CLIENT-IP'] = ip
        headers['X-FORWARDED-FOR'] = ip
    headers = headers.copy()
    headers.update(header)
    if proxy_ip != "":
        proxy_ip_dict = {
            "http": proxy_ip,
            "https": proxy_ip
        }
    else:
        proxy_ip_dict = dict()
    s = requests.session()
    if isinstance(value, dict):
        s.post(url, data=value, headers=headers, proxies=proxy_ip_dict, verify=False, timeout=timeout,
               allow_redirects=allow_redirects)
    elif isinstance(params, dict):
        s.get(url, headers=headers, proxies=proxy_ip_dict, verify=False, timeout=timeout, params=params,
              allow_redirects=allow_redirects)
    else:
        s.get(url, headers=headers, proxies=proxy_ip_dict, verify=False, timeout=timeout,
              allow_redirects=allow_redirects)
    try:
        c = s.cookies.RequestsCookieJar()
        c.set('cookie-name', 'cookie-value')
        s.cookies.update(c)
    except:
        pass
    return s.cookies.get_dict()


# noinspection PyBroadException
def get_client_ip(request):
    """
    This function can get client ip.

    :param request: client request
    :return: str
    """
    try:
        real_ip = request.META['HTTP_X_FORWARDED_FOR']
        regip = real_ip.split(",")[0]
    except Exception:
        try:
            regip = request.META['REMOTE_ADDR']
        except:
            regip = ""
    return regip


def byte_to_str(data, data_type=None):
    """
    This function is born for happybase's data.
    Because happybase's data is all bytes.
    We should turn it into str(s) which has/have been decode.

    :param data_type:
    :param data: generator or other type's data.
    :return: type of source data
    """
    if data_type == "generator":
        result = list()
        for v in data:
            result.append(byte_to_str(v))
        return result
    elif isinstance(data, tuple):
        result = list()
        for v in data:
            result.append(byte_to_str(v))
        return tuple(result)
    elif isinstance(data, bytes):
        try:
            return data.decode("utf-8")
        except UnicodeDecodeError:
            try:
                return data.decode("GBK")
            except UnicodeDecodeError:
                return None
    elif isinstance(data, dict):
        return dict(map(byte_to_str, data.items()))
    else:
        return data


def virtual_ip():
    """
    This function can get a virtual ip. We can use it to make a virtual X-FORWARDED-FOR or CLIENT-IP.
    :return: str : a virtual ip.
    """
    ip_list = [
        "218", "218", "66", "66", "218", "218", "60", "60", "202", "204", "66", "66", "66", "59", "61", "60", "222",
        "221",
        "66", "59", "60", "60", "66", "218", "218", "62", "63", "64", "66", "66", "122", "211"]
    randindex = random.randint(0, len(ip_list))
    if randindex > 30:
        randindex = randindex - 1
    ip_1 = ip_list[randindex]
    ip_2 = round(random.randint(600000, 2550000) / 10000)
    ip_3 = round(random.randint(600000, 2550000) / 10000)
    ip_4 = round(random.randint(600000, 2550000) / 10000)
    ip = str(ip_1) + "." + str(ip_2) + "." + str(ip_3) + "." + str(ip_4)
    return ip


def get_time_stamp(date, date_format):
    """
    get a timestamp which type is int
    :param date:
    :param date_format:
    :return:
    """
    time_arr = time.strptime(date, date_format)
    return int(time.mktime(time_arr))


def get_now_time_stamp():
    return int(time.time())


def get_date_time(time_stamp, date_format):
    """
    get a date which type is str
    :param time_stamp:
    :param date_format:
    :return:
    """
    time_arr = time.localtime(time_stamp)
    return time.strftime(date_format, time_arr)


def get_user_agent(type_get=1, index=False):
    """
    :param type_get: type 1 will return a random UserAgent in list
    type 2 will return a UserAgent which you choose, so you should input a index
    :param index: UserAgent's index
    :return:
    """
    # UserAgent array(list)
    agentArr = (
        "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/14.0.835.163 Safari/535.1",
        "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0) Gecko/20100101 Firefox/6.0",
        "Mozilla/5.0 (Windows NT 6.1; WOW64) AppleWebKit/534.50 (KHTML, like Gecko) Version/5.1 Safari/534.50",
        "Opera/9.80 (Windows NT 6.1; U; zh-cn) Presto/2.9.168 Version/11.50",
        "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; Win64; x64; Trident/5.0; .NET CLR 2.0.50727; SLCC2; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; Tablet PC 2.0; .NET4.0E)",
        "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; WOW64; Trident/4.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; .NET4.0C; InfoPath.3)",
        "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0; GTB7.0)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 5.1)",
        "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)",
        "Mozilla/5.0 (Windows; U; Windows NT 6.1; ) AppleWebKit/534.12 (KHTML, like Gecko) Maxthon/3.0 Safari/534.12",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E; SE 2.X MetaSr 1.0)",
        "Mozilla/5.0 (Windows; U; Windows NT 6.1; en-US) AppleWebKit/534.3 (KHTML, like Gecko) Chrome/6.0.472.33 Safari/534.3 SE 2.X MetaSr 1.0",
        "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E)",
        "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/535.1 (KHTML, like Gecko) Chrome/13.0.782.41 Safari/535.1 QQBrowser/6.9.11079.201",
        "Mozilla/4.0 (compatible; MSIE 7.0; Windows NT 6.1; WOW64; Trident/5.0; SLCC2; .NET CLR 2.0.50727; .NET CLR 3.5.30729; .NET CLR 3.0.30729; Media Center PC 6.0; InfoPath.3; .NET4.0C; .NET4.0E) QQBrowser/6.9.11079.201",
        "Mozilla/5.0 (compatible; MSIE 9.0; Windows NT 6.1; WOW64; Trident/5.0)"
    )
    if type_get == 1:
        if not isinstance(index, int):
            index = int((random.random()) * 1000) % len(agentArr)
        return agentArr[index]
    elif type_get == 2:
        return len(agentArr)



"""
author songjie
"""
from contextlib import contextmanager

from .function import debug


def default_callback(**kwargs):
    """
    HelperContext default callback
    :return:
    """
    pass


class HelperContext(object):
    def __init__(self):
        pass

    def __del__(self):
        pass

    @contextmanager
    def auto_handle_exception(self,
                              before_callback=default_callback,
                              error_callback=default_callback,
                              after_callback=default_callback,
                              throw_exception_flag=False,
                              **kwargs
                              ):
        try:
            before_callback(**kwargs)
            yield
            after_callback(**kwargs)
        except Exception as e:
            error_callback(**kwargs)
            if throw_exception_flag:
                debug(e)



"""
tool-yy
-------------

tool-yy is used to get some help and quickly connect the database from anywhere.

Links
`````

* `development version <https://github.com/guaidashu/python3_tool_myself>`_
"""
import os
import sys

from setuptools import setup, find_packages

about = {}

with open('tool_yy/__about__.py') as f:
    exec(f.read(), about)

if sys.argv[-1] == 'test':
    status = os.system('make check')
    status >>= 8
    sys.exit(status)

setup(
    name=about['__title__'],
    version=about['__version__'],
    author=about['__author__'],
    author_email=about['__author_email__'],
    description=about['__description__'],
    long_description=__doc__,
    long_description_content_type="text/markdown",
    license=about['__license__'],
    url=about['__url__'],
    packages=find_packages(),
    install_requires=about['__install_requires__'],
    classifiers=[
        "Topic :: Internet :: WWW/HTTP :: Dynamic Content",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ]
)



"""
Create by yy on 2019-07-25
"""
from .helper_config import HelperConfig


class Helper(HelperConfig):
    __slots__ = ("psql", "phoniex", "db")

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def set_config(self, config=None):
        self.config.from_object(config)
